@@ -1,17 +1,17 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2005-2020 by Dominique Dumont.
+# This software is Copyright (c) 2005-2022 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-package Config::Model 2.140;
+package Config::Model 2.152;
 
+use 5.20.0;
 use strict ;
 use warnings;
-use 5.12.0;
 
 use Mouse;
 use Mouse::Util::TypeConstraints;
@@ -29,11 +29,20 @@ use File::HomeDir;
 use Cwd;
 use Config::Model::Lister;
 
+with "Config::Model::Role::Constants";
+
 use parent qw/Exporter/;
 our @EXPORT_OK = qw/cme initialize_log4perl/;
 
-# this class holds the version number of the package
-use vars qw(@status @level %default_property $force_default_log);
+use feature qw/signatures postderef/;
+no warnings qw/experimental::signatures experimental::postderef/;
+
+# used in some tests where we don't want to load
+# ~/.log4config-model config
+my $force_default_log;
+sub force_usage_of_default_log_config () {
+    return $force_default_log = 1;
+}
 
 my $legacy_logger = get_logger("Model::Legacy") ;
 my $loader_logger = get_logger("Model::Loader") ;
@@ -43,13 +52,6 @@ my $logger = get_logger("Model") ;
 # cme function
 my $model_storage;
 
-%default_property = (
-    status      => 'standard',
-    level       => 'normal',
-    summary     => '',
-    description => '',
-);
-
 enum LegacyTreament => qw/die warn ignore/;
 
 has skip_include => ( isa => 'Bool',           is => 'ro', default => 0 );
@@ -74,6 +76,7 @@ has instances => (
 # element description in class or in element declaration)), this raw format is not
 # usable without normalization (done by normalize_class_parameters)
 
+# the key if this hash is a model name
 has raw_models => (
     isa     => 'HashRef',
     is      => 'ro',
@@ -83,16 +86,12 @@ has raw_models => (
         raw_model_exists  => 'exists',
         raw_model_defined => 'defined',
         raw_model         => 'get',
+        get_raw_model     => 'get',
         store_raw_model   => 'set',
         raw_model_names   => 'keys',
     },
 );
 
-sub get_raw_model {
-    my $self = shift;
-    return $self->raw_model(@_);
-}
-
 # the result of normalization is stored here. Normalized model aggregate user models and
 # augmented features (the one found in Foo.d directory). inclusion of other class is NOT
 # yet done. normalized_models are created while loading files (load method) or creating
@@ -159,20 +158,16 @@ has skip_inheritance => (
         $self->skip_include = $self->skip_inheritance;
     } );
 
-around BUILDARGS => sub {
-    my $orig  = shift;
-    my $class = shift;
-
-    my %args = @_;
+# remove this hack mid 2022
+around BUILDARGS => sub ($orig, $class, %args) {
     my %new;
     foreach my $k (keys %args) {
         if (defined $args{$k}) {
             $new{$k} = $args{$k};
         }
         else {
-            # this warning should be changed to an error end of 2017
             # cannot use logger, it's not initialised yet
-            warn("Config::Model new: passing undefined constructor argument is deprecated ($k argument)\n");
+            croak("Config::Model new: passing undefined constructor argument is deprecated ($k argument)\n");
         }
     }
 
@@ -181,13 +176,13 @@ around BUILDARGS => sub {
 
 # keep this as a separate sub from BUILD. So user can call it before
 # creating Config::Model object
-sub initialize_log4perl {
-    if (ref $_[0]) {
+sub initialize_log4perl (@args) {
+    if (ref $args[0]) {
         # may be called as $self-> initialize_log4perl
-        shift;
+        shift @args;
     }
 
-    my %args = @_;
+    my %args = @args;
 
     my $log4perl_syst_conf_file = path('/etc/log4config-model.conf');
     # avoid undef warning when homedir is not defined (e.g. with Debian cowbuilder)
@@ -224,6 +219,7 @@ sub BUILD {
     my $self = shift;
     my $args = shift;
     initialize_log4perl(verbose => $args->{verbose}) unless Log::Log4perl->initialized();
+    return;
 }
 
 sub show_legacy_issue {
@@ -241,6 +237,7 @@ sub show_legacy_issue {
     } elsif ( $behavior eq 'note' ) {
         $legacy_logger->info( @msg);
     }
+    return;
 }
 
 sub _tweak_instance_args {
@@ -275,10 +272,15 @@ sub _tweak_instance_args {
         || delete $args->{name}          # preferred parameter
         || $app_name                     # fallback in most cases
         || 'default';                    # fallback mostly in tests
+    return;
 }
 
-sub cme {
-    my %args = @_ == 1 ? ( application => $_[0]) : @_ ;
+sub cme (@args) {
+    my %args = @args == 1 ? ( application => $args[0]) : @args ;
+
+    if (my $force = delete $args{'force-load'}) {
+        $args{check} = 'no' if $force;
+    }
 
     my $cat =_tweak_instance_args(\%args);
 
@@ -286,12 +288,11 @@ sub cme {
     # model_storage is used to keep Config::Model object alive
     $model_storage //= Config::Model->new(%$m_args);
 
-    return $model_storage->instance(@_);
+    return $model_storage->instance(%args);
 }
 
-sub instance {
-    my $self = shift;
-    my %args = @_ == 1 ? ( application => $_[0]) : @_ ;
+sub instance ($self, @args) {
+    my %args = @args == 1 ? ( application => $args[0]) : @args ;
 
     # also creates a default name
     _tweak_instance_args(\%args);
@@ -323,10 +324,6 @@ sub instance_names {
     return @all;
 }
 
-@level = qw/hidden normal important/;
-
-@status = qw/obsolete deprecated standard/;
-
 # unpacked model is:
 # {
 #   element_list  => [ ... ],
@@ -351,10 +348,7 @@ my @other_legal_params = qw/ author element status description summary level acc
 
 # keep as external API. All internal call go through _store_model
 #  See comments around raw_models attribute for explanations
-sub create_config_class {
-    my $self      = shift;
-    my %raw_model = @_;
-
+sub create_config_class ($self, %raw_model) {
     my $config_class_name = delete $raw_model{name}
         or croak "create_config_class: no config class name";
 
@@ -431,6 +425,7 @@ sub include_backend {
             }
         }
     }
+    return;
 }
 
 sub normalize_class_parameters {
@@ -458,25 +453,6 @@ sub normalize_class_parameters {
         push @element_list, ref($item) ? @$item : ($item);
     }
 
-    # optional parameter to force element order. Useful when parameters declarations
-    # are grouped. Although interaction with include may be tricky. Let's not advertise it.
-    # yet.
-
-    if ( defined $normalized_model->{force_element_order} ) {
-        my @forced_list = @{ delete $normalized_model->{force_element_order} };
-        my %forced = map { ( $_ => 1 ) } @forced_list;
-        foreach (@element_list) {
-            next if delete $forced{$_};
-            Config::Model::Exception::ModelDeclaration->throw( error =>
-                    "class $config_class_name: element $_ is not in force_element_order list" );
-        }
-        if (%forced) {
-            Config::Model::Exception::ModelDeclaration->throw(
-                error => "class $config_class_name: force_element_order list has unknown elements "
-                    . join( ' ', keys %forced ) );
-        }
-    }
-
     if ( defined $normalized_model->{inherit_after} ) {
         $self->show_legacy_issue([ "Model $config_class_name: inherit_after is deprecated ",
             "in favor of include_after" ]);
@@ -600,6 +576,7 @@ sub handle_experience_permission {
     if (delete $model->{experience}) {
         carp "experience parameter is deprecated";
     }
+    return;
 }
 
 sub translate_legacy_info {
@@ -689,6 +666,7 @@ sub translate_legacy_info {
     $legacy_logger->debug(
         Data::Dumper->Dump( [$info], [ 'translated_' . $elt_name ] ) 
     ) if $legacy_logger->is_debug;
+    return;
 }
 
 sub translate_legacy_backend_info {
@@ -742,7 +720,7 @@ sub translate_legacy_backend_info {
         $self->show_legacy_issue("$config_class_name: backend $ref->{backend}: allow_empty is deprecated. Use auto_create", 'warn');
         $ref->{auto_create} = delete $ref->{allow_empty};
     }
-
+    return;
 }
 
 sub translate_cargo_info {
@@ -777,6 +755,7 @@ sub translate_cargo_info {
     $legacy_logger->debug( 
         Data::Dumper->Dump( [$info], [ 'translated_' . $elt_name ] ) 
     ) if $legacy_logger->is_debug;
+    return;
 }
 
 sub translate_id_names {
@@ -787,6 +766,7 @@ sub translate_id_names {
     $self->translate_name( $config_class_name, $elt_name, $info, 'allow',      'allow_keys',       'die' );
     $self->translate_name( $config_class_name, $elt_name, $info, 'allow_from', 'allow_keys_from',  'die' );
     $self->translate_name( $config_class_name, $elt_name, $info, 'follow',     'follow_keys_from', 'die' );
+    return;
 }
 
 sub translate_name {
@@ -799,6 +779,7 @@ sub translate_name {
         );
         $info->{$to} = delete $info->{$from};
     }
+    return;
 }
 
 sub translate_allow_compute_override {
@@ -813,6 +794,7 @@ sub translate_allow_compute_override {
         );
         $info->{compute}{allow_override} = delete $info->{allow_compute_override};
     }
+    return;
 }
 
 sub translate_compute_info {
@@ -855,6 +837,7 @@ sub translate_compute_info {
             Data::Dumper->Dump( [ $info->{$new_name} ], [ 'new_' . $new_name ] )
         ) if $legacy_logger->is_debug;
     }
+    return;
 }
 
 sub translate_id_class {
@@ -883,6 +866,7 @@ sub translate_id_class {
         "translate_id_class $elt_name output:",
         Data::Dumper->Dump( [$info], [qw/new_info/])
     ) if $legacy_logger->is_debug;
+    return;
 }
 
 # internal: translate default information for id element
@@ -918,6 +902,7 @@ sub translate_id_default_info {
         "translate_id_default_info $elt_name output:",
         Data::Dumper->Dump( [$info], [qw/new_info/])
     ) if $legacy_logger->is_debug;
+    return;
 }
 
 # internal: translate auto_create information for id element
@@ -953,6 +938,7 @@ sub translate_id_auto_create {
         "translate_id_default_info $elt_name output:\n",
         Data::Dumper->Dump( [$info], [qw/new_info/] )
     ) if $legacy_logger->is_debug;
+    return;
 }
 
 sub translate_id_min_max {
@@ -973,6 +959,7 @@ sub translate_id_min_max {
 
         $info->{$good} = delete $info->{$bad};
     }
+    return;
 }
 
 sub translate_warped_node_info {
@@ -999,6 +986,7 @@ sub translate_warped_node_info {
         "translate_warped_node_info $elt_name output:\n",
         Data::Dumper->Dump( [$info], [qw/new_info/] )
     ) if $legacy_logger->is_debug;
+    return;
 }
 
 # internal: translate warp information into 'boolean expr' => { ... }
@@ -1028,6 +1016,7 @@ sub translate_warp_info {
         "translate_warp_info $elt_name output:\n",
         Data::Dumper->Dump( [$warp_info], [qw/new_warp_info/] )
     ) if $legacy_logger->is_debug;
+    return;
 }
 
 # internal
@@ -1181,6 +1170,7 @@ sub translate_legacy_builtin {
 
     $legacy_logger->debug( Data::Dumper->Dump( [$model], ['translated_builtin'] )) 
         if $legacy_logger->is_debug;
+    return;
 }
 
 sub translate_legacy_built_in_list {
@@ -1200,6 +1190,7 @@ sub translate_legacy_built_in_list {
 
     $legacy_logger->debug( Data::Dumper->Dump( [$model], ['translated_built_in_list'] ))
         if $legacy_logger->is_debug;
+    return;
 }
 
 sub include_class {
@@ -1220,6 +1211,7 @@ sub include_class {
     foreach my $inc ( reverse @includes ) {
         $self->include_one_class( $class_name, $target_model, $inc, $include_after );
     }
+    return;
 }
 
 sub include_one_class {
@@ -1281,46 +1273,73 @@ sub include_one_class {
         }
     }
     get_logger('Model')->debug("class $class_name include $include_class done");
+    return;
 }
 
-# load a model from file. See comments around raw_models attribute for explanations
-sub load {
-    my $self       = shift;
-    my $model_name = shift;    # model name like Foo::Bar
-    my $load_file  = shift;    # model file (override model name), used for tests
-
-    $loader_logger->debug("called on model $model_name");
-
-    my $load_path = $model_name;
-    $load_path =~ s/::/\//g;
-
-    $load_file ||= $self->model_dir . '/' . $load_path . '.pl';
-
-    $loader_logger->debug("model $model_name from file $load_file");
-
-    my %models_by_name;
-
-    # Searches $load_file in @INC and returns an array containing the
-    # names of the loaded clases
-    my @loaded_classes = $self->_load_model_in_hash( \%models_by_name, $load_file );
-
-    $self->store_raw_model( $model_name, dclone( \%models_by_name ) );
-
-    foreach my $name ( keys %models_by_name ) {
-        my $data = $self->normalize_class_parameters( $name, $models_by_name{$name} );
-        $loader_logger->debug("Store normalized model $name");
-        $self->store_normalized_model( $name, $data );
+sub find_model_file_in_dir ($model_name, $model_path) {
+    foreach my $ext (qw/yml yaml pl/) {
+        my $sub_path = $model_name =~ s!::!/!rg;
+        my $path_load_file = $model_path->child($sub_path . '.' . $ext);
+        return $path_load_file if $path_load_file->exists;
     }
+    return;
+}
+
+sub find_model_file_in_inc {
+    my ($self, $model_name, $load_file) = @_;
+
+    my $path_load_file ;
+
+    if ($load_file and $load_file =~ m!^/! ) {
+        # load_file is absolute, do not search in @INC
+        $path_load_file = $load_file;
+    } elsif ($self->model_dir and $self->model_dir =~ m!^/!) {
+        # model_dir is absolute, do not search in @INC
+        my $model_path = path($self->model_dir);
+        $path_load_file = find_model_file_in_dir ($model_name, $model_path);
+        Config::Model::Exception::ModelDeclaration->throw(
+            error => "Cannot find $model_name file in $model_path"
+        ) unless $path_load_file;
+    }
+    else {
+        foreach my $inc_str (@INC) {
+            my $inc_path = path($inc_str);
+            if ($load_file) {
+                $path_load_file = $inc_path->child($load_file);
+            }
+            else {
+                my $sub_path = $model_name =~ s!::!/!rg;
+                my $model_path = $inc_path->child($self->model_dir);
+                foreach my $ext (qw/yml yaml pl/) {
+                    $path_load_file = $model_path->child($sub_path . '.' . $ext);
+                    last if $path_load_file->exists;
+                }
+            }
+            last if $path_load_file->exists;
+        }
+    }
+
+    Config::Model::Exception::ModelDeclaration->throw(
+        error => "Cannot find $model_name file in \@INC"
+    ) unless $path_load_file;
+
+    $loader_logger->debug("model $model_name from file $path_load_file");
+
+    return $path_load_file;
+}
+
+sub load_model_plugins {
+    my ($self, @model_names) = @_;
 
     # look for additional model information
     my %model_graft_by_name;
     my %done;  # avoid loading twice the same snippet (where system version may clobber dev version)
 
     foreach my $inc_str (@INC) {
-        foreach my $name ( keys %models_by_name ) {
+        foreach my $name ( @model_names ) {
             my $snippet_path = $name;
             $snippet_path =~ s/::/\//g;
-            my $snippet_dir = path($inc_str)->child($self->model_dir)->child($snippet_path . '.d');
+            my $snippet_dir = path($inc_str)->absolute->child($self->model_dir)->child($snippet_path . '.d');
             $loader_logger->trace("looking for snippet in $snippet_dir");
             if ( $snippet_dir->is_dir ) {
                 my $iter = $snippet_dir->iterator({ recurse => 1 });
@@ -1339,13 +1358,43 @@ sub load {
 
                     my $done_key = $name . ':' . $snippet_file_rel;
                     next if $done{$done_key};
-                    $loader_logger->info("Found snippet $snippet_file_rel in $inc_str dir");
-                    $self->_load_model_in_hash( \%model_graft_by_name, $snippet_file_rel);
+                    $loader_logger->info("Found snippet $snippet_file in $inc_str dir");
+                    my $snippet_model = $self->_load_model_file($snippet_file);
+
+                    $self->_merge_model_in_hash( \%model_graft_by_name, $snippet_model, $snippet_file_rel);
                     $done{$done_key} = 1;
                 }
             }
         }
     }
+    return %model_graft_by_name;
+}
+
+# load a model from file. See comments around raw_models attribute for explanations
+sub load {
+    my $self       = shift;
+    my $model_name = shift;    # model name like Foo::Bar
+    my $load_file  = shift;    # model file (override model name), used for tests
+
+    $loader_logger->debug("called on model $model_name");
+    my $path_load_file = $self->find_model_file_in_inc($model_name, $load_file);
+
+    my %models_by_name;
+
+    # Searches $load_file in @INC and returns an array containing the
+    # names of the loaded classes
+    my $model = $self->_load_model_file($path_load_file->absolute);
+    my @loaded_classes = $self->_merge_model_in_hash( \%models_by_name, $model, $path_load_file );
+
+    $self->store_raw_model( $model_name, dclone( \%models_by_name ) );
+
+    foreach my $name ( keys %models_by_name ) {
+        my $data = $self->normalize_class_parameters( $name, $models_by_name{$name} );
+        $loader_logger->debug("Store normalized model $name");
+        $self->store_normalized_model( $name, $data );
+    }
+
+    my %model_graft_by_name = $self->load_model_plugins(sort keys %models_by_name);
 
     # store snippet. May be used later
     foreach my $name (keys %model_graft_by_name) {
@@ -1369,10 +1418,8 @@ sub load {
 
 # New subroutine "_load_model_in_hash" extracted - Fri Apr 12 17:29:56 2013.
 #
-sub _load_model_in_hash {
-    my ( $self, $hash_ref, $load_file ) = @_;
-
-    my $model = $self->_do_model_file($load_file);
+sub _merge_model_in_hash {
+    my ( $self, $hash_ref, $model, $load_file ) = @_;
 
     my @names;
     foreach my $config_class_info (@$model) {
@@ -1391,10 +1438,7 @@ sub _load_model_in_hash {
     return @names;
 }
 
-#
-# New subroutine "_do_model_file" extracted - Sun Nov 28 17:25:35 2010.
-#
-sub _do_model_file {
+sub _load_model_file {
     my ( $self, $load_file ) = @_;
 
     $loader_logger->info("load model $load_file");
@@ -1428,6 +1472,7 @@ sub augment_config_class {
         || croak "augment_config_class: missing class name";
 
     $self->augment_config_class_really( $config_class_name, \%augment_data );
+    return;
 }
 
 sub augment_config_class_really {
@@ -1454,6 +1499,7 @@ sub augment_config_class_really {
     }
 
     $self->store_normalized_model( $config_class_name => $new_model );
+    return;
 }
 
 sub model {
@@ -1479,7 +1525,7 @@ sub model {
 sub get_model {
     my ($self,$model) = @_;
     carp "get_model is deprecated in favor of get_model_clone";
-    $self->get_model_clone($model);
+    return $self->get_model_clone($model);
 }
 
 sub get_model_clone {
@@ -1730,6 +1776,7 @@ sub generate_doc {
             print $res->{$class_name};
         }
     }
+    return;
 }
 
 sub get_element_model {
@@ -1763,10 +1810,7 @@ sub get_normalized_model {
     return dclone($normalized_model);
 }
 
-sub get_element_name {
-    my $self = shift;
-    my %args = @_;
-
+sub get_element_name ($self, %args) {
     my $class = $args{class}
         || croak "get_element_name: missing 'class' parameter";
 
@@ -1782,17 +1826,14 @@ sub get_element_name {
     # the user
     foreach my $elt ( @{ $model->{element_list} } ) {
         my $elt_data = $model->{element}{$elt};
-        my $l = $elt_data->{level} || $default_property{level};
+        my $l = $elt_data->{level} || get_default_property('level');
         push @result, $elt if $l ne 'hidden' ;
     }
 
     return wantarray ? @result : join( ' ', @result );
 }
 
-sub get_element_property {
-    my $self = shift;
-    my %args = @_;
-
+sub get_element_property ($self, %args) {
     my $elt = $args{element}
         || croak "get_element_property: missing 'element' parameter";
     my $prop = $args{property}
@@ -1806,13 +1847,13 @@ sub get_element_property {
     if ( not defined $model->{element}{$elt} ) {
         $logger->debug("test accept for class $class elt $elt prop $prop");
         foreach my $acc_re ( @{ $model->{accept_list} } ) {
-            return $model->{accept}{$acc_re}{$prop} || $default_property{$prop}
+            return $model->{accept}{$acc_re}{$prop} || get_default_property($prop)
                 if $elt =~ /^$acc_re$/;
         }
     }
 
     return $self->model($class)->{element}{$elt}{$prop}
-        || $default_property{$prop};
+        || get_default_property($prop);
 }
 
 sub list_class_element {
@@ -1862,7 +1903,7 @@ Config::Model - a framework to validate, migrate and edit configuration files
 
 =head1 VERSION
 
-version 2.140
+version 2.152
 
 =head1 SYNOPSIS
 
@@ -2459,8 +2500,15 @@ Check if an instance name already exists
 =head2 cme
 
 This method is syntactic sugar for short program. It creates a new
-C<Config::Model> object and returns a new instance. See L</instance>
-for the parameters.
+C<Config::Model> object and returns a new instance.
+
+C<cme> arguments are passed to L</instance> method, except
+C<force-load>.
+
+Like L<cme> command, C<cme> functions accepts C<force-load>
+parameters. When this argument is true, the instance is created with
+C<check => 'no'>. Hence bad values are stored in C<cme> and must be
+corrected before saving back the data.
 
 =head1 Configuration class
 
@@ -2737,7 +2785,7 @@ Dominique Dumont, "ddumont@cpan.org"
 
 =head1 CREDITS
 
-In alphabetical order:
+Contributors to this project are listed in alphabetical order:
 
   Harley Pig
 
@@ -2751,24 +2799,9 @@ In alphabetical order:
 
   Mohammad S Anwar
 
-=head1 LICENSE
+  Topi Miettinen
 
-    Copyright (c) 2005-2016 Dominique Dumont.
-
-    Config-Model is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public License as
-    published by the Free Software Foundation; either version 2.1 of
-    the License, or (at your option) any later version.
-
-    Config-Model is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with Config-Model; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-    02110-1301 USA
+Many thanks for your help
 
 =head1 SEE ALSO
 
@@ -2926,7 +2959,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2005-2020 by Dominique Dumont.
+This software is Copyright (c) 2005-2022 by Dominique Dumont.
 
 This is free software, licensed under:
 
