@@ -7,7 +7,7 @@
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-package Config::Model::Node 2.149;
+package Config::Model::Node 2.152;
 
 use Mouse;
 with "Config::Model::Role::NodeLoader";
@@ -1129,7 +1129,7 @@ sub apply_fixes ($self, $filter='' ) {
     $fix_logger->debug( "apply fix started from ", $self->name );
     $scan->scan_node( undef, $self );
     $fix_logger->trace("apply fix done");
-    return;
+    return $self;
 }
 
 sub deep_check ($self, %args){
@@ -1181,7 +1181,7 @@ Config::Model::Node - Class for configuration tree node
 
 =head1 VERSION
 
-version 2.149
+version 2.152
 
 =head1 SYNOPSIS
 
@@ -1835,7 +1835,7 @@ deprecated elements or values. Return 1 if data needs to be saved.
 =head2 apply_fixes
 
 Scan the tree from this node and apply fixes that are attached to warning specifications.
-See C<warn_if_match> or C<warn_unless_match> in L<Config::Model::Value/>.
+See C<warn_if_match> or C<warn_unless_match> in L<Config::Model::Value/>. Return C<$self> since v2.151.
 
 =head2 load
 
